@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt.h,v 1.151 2007/09/26 15:28:53 remko Exp $
+ *	$Id: gmt.h,v 1.174 2009/01/09 04:02:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -78,8 +78,10 @@ extern "C" {
 #include <time.h>
 
 #ifdef __FreeBSD__
+#ifdef _i386_
 #include <floatingpoint.h>
 #endif
+#endif
 
 #include "gmt_notunix.h"	/* Stuff for Windows, OS/2 etc */
 #ifndef WIN32
@@ -119,7 +121,7 @@ extern "C" {
 #define	M_LN2_INV	(1.0 / 0.69314718055994530942)
 #endif
 #ifndef M_EULER
-#define M_EULER 0.577215664901532860606512	/* Euler's contant (gamma) */
+#define M_EULER 0.577215664901532860606512	/* Euler's constant (gamma) */
 #endif
 
 #define GMT_CONV_LIMIT	1.0e-8	/* Fairly tight convergence limit or "close to zero" limit */
@@ -181,14 +183,26 @@ extern "C" {
 
 /* Safe math macros that check arguments */
 
+#ifdef __APPLE__
+#ifdef __x86_64__
+/* Temorarily bypass Apple 64-bit log10 bug on Leopard 10.5.2 */
+/* #define LOG2_10_SCALE 0x1.34413509F79FFp-2 */
+#define LOG2_10_SCALE 0.30102999566398114250631579125183634459972381591796875
+#define d_log10(x) ((x) <= 0.0 ? GMT_d_NaN : LOG2_10_SCALE * log2 (x))
+#define d_log101p(x) ((x) <= -1.0 ? GMT_d_NaN : LOG2_10_SCALE * log2 (1.0+(x)))
+#endif
+#endif
+#ifndef LOG2_10_SCALE
+#define d_log10(x) ((x) <= 0.0 ? GMT_d_NaN : log10 (x))
+#define d_log101p(x) ((x) <= -1.0 ? GMT_d_NaN : log10 (1.0+(x)))
+#endif
 #define d_sqrt(x) ((x) < 0.0 ? 0.0 : sqrt (x))
 #define d_acos(x) (fabs (x) >= 1.0 ? ((x) < 0.0 ? M_PI : 0.0) : acos (x))
 #define d_asin(x) (fabs (x) >= 1.0 ? copysign (M_PI_2, (x)) : asin (x))
 #define d_atan2(y,x) ((x) == 0.0 && (y) == 0.0 ? 0.0 : atan2 (y, x))
 #define d_log(x) ((x) <= 0.0 ? GMT_d_NaN : log (x))
-#define d_log10(x) ((x) <= 0.0 ? GMT_d_NaN : log10 (x))
 #define d_log1p(x) ((x) <= -1.0 ? GMT_d_NaN : log1p (x))
-#define d_log101p(x) ((x) <= -1.0 ? GMT_d_NaN : log10 (1.0+(x)))
+
 
 /* Macros for degree-based trig */
 
@@ -199,6 +213,7 @@ extern "C" {
 
 /* Macros for swapping misc data types */
 
+#define l_swap(x, y) {GMT_LONG tmp; tmp = x, x = y, y = tmp;}
 #define i_swap(x, y) {int tmp; tmp = x, x = y, y = tmp;}
 #define d_swap(x, y) {double tmp; tmp = x, x = y, y = tmp;}
 #define f_swap(x, y) {float tmp; tmp = x, x = y, y = tmp;}
@@ -237,29 +252,24 @@ extern "C" {
  *			GMT TYPEDEF DEFINITIONS
  *--------------------------------------------------------------------*/
 
-typedef int BOOLEAN;		/* BOOLEAN used for logical variables */
-typedef void (*PFV) ();		/* PFV declares a pointer to a function returning void */
-typedef int (*PFI) ();		/* PFI declares a pointer to a function returning an int */
-typedef BOOLEAN (*PFB) ();	/* PFB declares a pointer to a function returning a BOOLEAN */
-typedef double (*PFD) ();	/* PFD declares a pointer to a function returning a double */
-typedef double GMT_dtime;	/* GMT internal time representation */
 #ifndef _WIN64
 typedef long GMT_LONG;		/* A signed 4 (or 8-byte for 64-bit) integer */
 #endif
+typedef int BOOLEAN;		/* BOOLEAN used for logical variables */
+typedef void (*PFV) ();		/* PFV declares a pointer to a function returning void */
+typedef GMT_LONG (*PFL) ();		/* PFI declares a pointer to a function returning an GMT_LONG */
+typedef int (*PFI) ();		/* PFI declares a pointer to a function returning an int */
+typedef BOOLEAN (*PFB) ();	/* PFB declares a pointer to a function returning a BOOLEAN */
+typedef double (*PFD) ();	/* PFD declares a pointer to a function returning a double */
 /*--------------------------------------------------------------------
  *			GMT PARAMETERS DEFINITIONS
  *--------------------------------------------------------------------*/
 
-#define GMT_N_KEYS 122		/* Number of gmt defaults */
-#define GMT_HASH_SIZE 122		/* Used in get_gmtdefaults, should be ~> GMT_N_KEYS */
 /* This structure contains default parameters for the GMT system */
 
-#define GMT_N_ELLIPSOIDS	64
-#define GMT_N_DATUMS	223
-
+#include "gmt_dimensions.h"
 #define GMT_PEN_LEN	128
 #define GMT_PENWIDTH	0.25	/* Default pen width in points */
-#define GMT_N_PEN_NAMES	12
 
 struct GMT_PEN {	/* Holds pen attributes */
 	double width;	/* In points */
@@ -268,8 +278,23 @@ struct GMT_PEN {	/* Holds pen attributes */
 	char texture[GMT_PEN_LEN];	/* In points */
 };
 
+struct GMT_TIME_SYSTEM {
+	char epoch[GMT_TEXT_LEN];	/* User-defined epoch for time */
+	char unit;			/* User-defined time unit */
+	GMT_LONG rata_die;		/* Rata die number of epoch */
+	double epoch_t0;		/* Rata_die fraction (in days since epoch, 0 <= t0 < 1) */
+	double scale;			/* Converts user units to seconds */
+	double i_scale;			/* Converts seconds to user units (= 1.0/scale) */
+};
+
+struct GMT_TIME_LANGUAGE {		/* Language-specific text strings for calendars */
+	char month_name[3][12][16];	/* Full, short, and 1-char month names */
+	char day_name[3][7][16];	/* Full, short, and 1-char weekday names */
+	char week_name[3][16];		/* Full, short, and 1-char versions of the word Week */
+};
+
 struct GMT_DEFAULTS {
-	/* DO NOT MAKE CHANGES HERE WITHOUT CORRESPONDING CHANGES TO gmt_defaults.h !!!!!!!!!!!!!!!!!!! */
+	/* DO NOT MAKE CHANGES HERE WITHOUT CORRESPONDING CHANGES TO gmt_globals.h !!!!!!!!!!!!!!!!!!! */
 	double annot_min_angle;		/* If angle between map boundary and annotation is less, no annotation is drawn [20] */
 	double annot_min_spacing;	/* If an annotation is closer that this to an older annotation, the annotation is skipped [0.0] */
 	int annot_font[2];		/* Font for primary and secondary annotations [Helvetica] */
@@ -318,7 +343,7 @@ struct GMT_DEFAULTS {
 	int oblique_annotation;		/* Controls annotations and tick angles etc. [0] */
 	int page_rgb[3];		/* Color of the page [255/255/255 white] */
 	BOOLEAN portrait;		/* Orientation of page [FALSE = Landscape, TRUE = Portrait] */
-	int paper_width[2];		/* Width and height of paper to plot on in points [Letter or A4] */
+	double paper_width[2];		/* Width and height of paper to plot on in points [Letter or A4] */
 	double polar_cap[2];		/* Latitude of polar cap and delta_lon for gridline spacing [85/90] */
 	BOOLEAN ps_colormode;		/* 2 writes HSV in PostScript, 1 writes CMYK, 0 uses RGB [0] */
 	int ps_compress;		/* Compression of PostScript images: 0 = no, 1 = RLE, 2 = LZW [0] */
@@ -330,7 +355,9 @@ struct GMT_DEFAULTS {
 	double tick_length;		/* Length of tickmarks [0.075] */
 	struct GMT_PEN tick_pen;	/* Pen attributes for tickmarks [2] */
 	BOOLEAN unix_time;		/* Plot time and map projection on map [FALSE] */
+	int unix_time_just;		/* Justification of the GMT timestamp box [1 (BL)] */
 	double unix_time_pos[2];	/* Where to plot timestamp relative to origin */
+	char unix_time_format[GMT_LONG_TEXT];	/* Specify the format for writing time stamps (see strftime) */
 	double vector_shape;		/* 0.0 = straight vectorhead, 1.0 = arrowshape, with continuous range in between */
 	BOOLEAN verbose;		/* Give info during execution [FALSE] */
 	BOOLEAN want_euro_font;		/* Include re-encoding for European characters [TRUE] */
@@ -366,9 +393,7 @@ struct GMT_DEFAULTS {
 	BOOLEAN time_is_interval;	/* Does a time given as a month (or year or day) mean the middle of the interval? */
 	double time_interval_fraction;	/* How much of a partial interval is needed in order to annotate it */
 	BOOLEAN want_leap_seconds;	/* Do we need to worry about leap seconds? */
-	char time_epoch[GMT_TEXT_LEN];	/* User-defined epoch for time */
-	char time_unit;			/* User-defined time unit */
-	int time_system;		/* Which time system is in effect */
+	struct GMT_TIME_SYSTEM time_system;	/* All the information about the selected time system */
 	int time_week_start;		/* Which day (Sun = 0, Sat = 7) is start of week */
 	char time_language[GMT_TEXT_LEN];	/* Language file for time support */
 	int Y2K_offset_year;		/* Cutoff for making 4-digit years from 2-digit years (1900 vs 2000) */
@@ -376,7 +401,7 @@ struct GMT_DEFAULTS {
 	enum gmt_symbol { gmt_none = -1, gmt_ring, gmt_degree, gmt_colon, gmt_squote, gmt_dquote, gmt_lastsym } degree_symbol;
 	struct gmt_encoding
 	{
-		char *name;
+		char name[GMT_TEXT_LEN];
 		int code[gmt_lastsym]; /* Codes for symbols we print. */
 	} encoding;
 	BOOLEAN history;		/* TRUE to pass information via .gmtdefaults4 files */
@@ -401,13 +426,14 @@ struct GMT_PS {	/* Holds the current settings that affect PS generation */
 	int line_join;				/* 0 (miter), 1 (round), 2 (bevel) */
 	int miter_limit;			/* 0-180 degrees as whole integer */
 	int dpi;				/* Plotter resolution in dots-per-inch */
-	int paper_width[2];			/* Physical width and height of paper used in points */
-	int page_rgb[3];			/* array with Color of page (paper) */
+	int page_rgb[3];			/* Array with Color of page (paper) */
+	int unix_time_just;			/* Justification of the GMT time stamp box */
+	double paper_width[2];			/* Physical width and height of paper used in points */
 	double x_origin, y_origin;		/* Result of -X -Y [gmtdefs.x|y_origin] */
 	double x_scale, y_scale;		/* Copy of gmtdefs.global_x|y_scale */
 	double unix_time_pos[2];		/* Result of -U [gmtdefs.unix_time_pos] */
-	char *encoding_name;			/* POinter to font encoding used */
-	char unix_time_label[BUFSIZ];		/* was gmtdefs.unix_time_label */
+	char *encoding_name;			/* Pointer to font encoding used */
+	char unix_time_label[BUFSIZ];		/* Label added to GMT time stamp generated by -U */
 };
 
 struct GMT_HASH {	/* Used to related keywords to gmtdefaults entry */
@@ -416,24 +442,6 @@ struct GMT_HASH {	/* Used to related keywords to gmtdefaults entry */
 	char *key;
 };
 
-struct GMT_TIME_SYSTEM {
-	char name[GMT_TEXT_LEN];	/* Name of system */
-	char epoch[GMT_TEXT_LEN];	/* Epoch time string */
-	char unit;			/* Time unit */
-#ifndef OLDCAL
-	int rata_die;			/* Rata die number of epoch */
-#endif
-	double epoch_t0;		/* epoch in seconds since start of rata_die */
-	double scale;			/* Converts user units to seconds */
-	double i_scale;			/* Converts seconds to user units (1.0/scale) */
-};
-
-struct GMT_TIME_LANGUAGE {	/* Language-specific text strings for calendars */
-	char month_name[3][12][16];	/* Full, short, and 1-char month names */
-	char day_name[3][7][16];	/* Full, short, and 1-char weekday names */
-	char week_name[3][16];		/* Full, short, and 1-char versions of the word Week */
-};
-
 struct GMT_FILL {	/* Holds fill attributes */
 	BOOLEAN use_pattern;	/* TRUE if pattern rather than rgb is set */
 	int rgb[3];		/* Chosen color if no pattern */
@@ -463,8 +471,10 @@ struct GMT_MAP_SCALE {	/* Used to plot a map scale in psbasemap and pscoast */
 	BOOLEAN plot;		/* TRUE if we want to draw the scale */
 	BOOLEAN fancy;		/* TRUE for a fancy map scale */
 	BOOLEAN gave_xy;	/* TRUE if x0, y0 was given in cartesian map coordinates and not lon/lat */
+	BOOLEAN unit;		/* TRUE if we should append distance unit to all annotations along the scale */
+	BOOLEAN do_label;	/* TRUE if we should plot a label for the scale */
 	char measure;		/* The unit, i.e., m (miles), n (nautical miles), or k (kilometers) */
-	char justify;		/* Placement of label: t(op), b(ottom), l(eft), r(ight), u(nit) */
+	char justify;		/* Placement of label: t(op), b(ottom), l(eft), r(ight) */
 	char label[GMT_TEXT_LEN];	/* Alternative user-specified label */
 	struct GMT_FILL fill;	/* Fill to use for background rectangle */
 	struct GMT_PEN pen;	/* Pen to use for background rectangle */
@@ -506,11 +516,11 @@ EXTERN_MSC char *GMT_USERDIR;			/* Points to the GMT user .gmt directory [~/.gmt
 EXTERN_MSC char *GMT_DATADIR;			/* Points to the GMT misc directory [if set] */
 EXTERN_MSC char *GMT_GRIDDIR;			/* Points to the GMT grids directory [if set] */
 EXTERN_MSC char *GMT_IMGDIR;			/* Points to the GMT img directory [if set] */
+EXTERN_MSC char *GMT_TMPDIR;			/* Points to the GMT temporary directory [if set] */
 EXTERN_MSC char *GMT_unit_names[];
 EXTERN_MSC double GMT_u2u[4][4];		/* measure unit translation matrix 4 x 4*/
 EXTERN_MSC struct GMT_FONT *GMT_font;
 EXTERN_MSC struct GMT_HASH GMT_month_hashnode[12];
-EXTERN_MSC struct GMT_TIME_SYSTEM GMT_time_system[];
 EXTERN_MSC struct GMT_TIME_LANGUAGE GMT_time_language;
 
 EXTERN_MSC float GMT_f_NaN;		/* Holds IEEE not-a-number float */
@@ -546,8 +556,8 @@ EXTERN_MSC struct GMT_TRUNCATE_TIME GMT_truncate_time;	/* Used to round off time
 EXTERN_MSC double *GMT_x_plot;			/* Holds the x/y (inches) of a line to be plotted */
 EXTERN_MSC double *GMT_y_plot;
 EXTERN_MSC int *GMT_pen;			/* Pen (3 = up, 2 = down) for these points */
-EXTERN_MSC int GMT_n_plot;			/* Number of such points */
-EXTERN_MSC int GMT_n_alloc;			/* Current size of allocated arrays */
+EXTERN_MSC GMT_LONG GMT_n_plot;			/* Number of such points */
+EXTERN_MSC GMT_LONG GMT_n_alloc;			/* Current size of allocated arrays */
 EXTERN_MSC int GMT_x_status_new;		/* Tells us what quadrant old and new points are in */
 EXTERN_MSC int GMT_y_status_new;
 EXTERN_MSC int GMT_x_status_old;
@@ -577,8 +587,8 @@ EXTERN_MSC BOOLEAN GMT_meridian_straight;	/* TRUE if meridians plot as straight
 EXTERN_MSC BOOLEAN GMT_parallel_straight;	/* TRUE if parallels plot as straight lines */
 EXTERN_MSC int GMT_3D_mode;			/* Determines if we draw fore and/or back 3-D box lines */
 EXTERN_MSC char *GMT_plot_format[3][2];		/* Keeps the 6 formats for dd:mm:ss plot output */
-EXTERN_MSC int GMT_n_lon_nodes;			/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
-EXTERN_MSC int GMT_n_lat_nodes;			/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
+EXTERN_MSC GMT_LONG GMT_n_lon_nodes;			/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
+EXTERN_MSC GMT_LONG GMT_n_lat_nodes;			/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
 EXTERN_MSC double GMT_dlon;			/* Steps taken in longitude along gridlines (gets reset in gmt_init.c) */
 EXTERN_MSC double GMT_dlat;			/* Steps taken in latitude along gridlines (gets reset in gmt_init.c) */
 
@@ -619,6 +629,12 @@ EXTERN_MSC PFD GMT_scan_time_string;		/*	pointer to functions that converts time
 #include "gmt_support.h"      	/* extern functions defined in gmt_support.c */
 #include "gmt_vector.h"      	/* extern functions defined in gmt_vector.c */
 
+#ifdef DEBUG
+#define GMT_memory(array,n,size,program) GMT_memory_func(array,n,size,program,__FILE__,__LINE__)
+#define GMT_free(array) GMT_free_func(array,__FILE__,__LINE__)
+extern struct MEMORY_TRACKER *GMT_mem_keeper;
+#endif
+
 #endif  /* _GMT_H */
 #ifdef __cplusplus
 }
