@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2xyz.c,v 1.51 2008/03/24 08:58:31 guru Exp $
+ *	$Id: grd2xyz.c,v 1.54 2009/01/09 04:02:33 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -203,9 +203,6 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[k]);
 
-		nm = ((size_t)grd.nx) * ((size_t)grd.ny);
-		n_total += nm;
-
 		if (e > w && n > s) {	/* Subset */
 			if (fabs (grd.x_max - grd.x_min) < 360.0) {
 				if (w < grd.x_min) { w = grd.x_min; error = TRUE; }
@@ -218,16 +215,15 @@ int main (int argc, char **argv)
 			GMT_err_fail (GMT_adjust_loose_wesn (&w, &e, &s, &n, &grd), "");	/* Make sure w,e,s,n matches header spacing */
 			nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
 			ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
-
-			z = (float *) GMT_memory (VNULL, (size_t) ((size_t)nx * (size_t)ny), sizeof (float), GMT_program);
-
-			GMT_err_fail (GMT_read_grd (argv[k], &grd, z, w, e, s, n, GMT_pad, FALSE), argv[k]);
+			nm = GMT_get_nm (nx, ny);
 		}
-		else {
-			z = (float *) GMT_memory (VNULL, (size_t) nm, sizeof (float), GMT_program);
+		else
+			nm = GMT_get_nm (grd.nx, grd.ny);
 
-			GMT_err_fail (GMT_read_grd (argv[k], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), argv[k]);
-		}
+		z = (float *) GMT_memory (VNULL, (size_t) nm, sizeof (float), GMT_program);
+		GMT_err_fail (GMT_read_grd (argv[k], &grd, z, w, e, s, n, GMT_pad, FALSE), argv[k]);
+
+		n_total += nm;
 
 		GMT_err_fail (GMT_set_z_io (&io, &grd), argv[k]);
 
@@ -246,8 +242,10 @@ int main (int argc, char **argv)
 			}
 		}
 		else if (Ctrl->E.active) {
-			if (grd.x_inc != grd.y_inc) {
-				fprintf (stderr, "%s: ERROR: x_inc and y_inc should be the same when writing to ESRI format\n", GMT_program);
+			double slop;
+			slop = 1.0 - (grd.x_inc / grd.y_inc);
+			if (!GMT_IS_ZERO (slop)) {
+				fprintf (stderr, "%s: ERROR: x_inc must equal y_inc when writing to ESRI format\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 			fprintf (GMT_stdout, "ncols %d\nnrows %d\n", grd.nx, grd.ny);
