@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdview.c,v 1.77 2007/10/03 00:35:13 guru Exp $
+ *	$Id: grdview.c,v 1.100 2009/01/12 00:07:13 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -98,7 +98,7 @@ struct GRDVIEW_CTRL {
 	struct W {	/* -W[+]<type><pen> */
 		BOOLEAN active;
 		BOOLEAN contour;
-		struct GMT_PEN pen[2];
+		struct GMT_PEN pen[3];
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
@@ -129,11 +129,13 @@ int main (int argc, char **argv)
 	char *topofile;
 	char *c_method[2] = {"colorimage", "colortiles",};
 
-	int 	i, j, ij, n_edges, k, k1, n, max, i_bin, j_bin, i_bin_old, j_bin_old;
-	int	side, way, nm, nm2, nx_f, ny_f, nx, ny, n_drape = 0, nm_use;
-	int	bin, two, mx, my, sw, se, nw, ne, id, n4, nk, c, rgb[3], i_start, i_stop;
+	int 	j, n_edges, max, i_bin, j_bin, i_bin_old, j_bin_old;
+	int	side, way, nx_f, ny_f, nx, ny;
+	int	two, sw, se, nw, ne, id, n4, nk, c, rgb[3], i_start, i_stop;
 	int	j_start, j_stop, i_inc, j_inc, q_set = 0, n_commas, ii, jj, n_out;
-	int	PS_colormask_off = 0, PS_colormask,*edge, bin_inc[4], ij_inc[4];
+	int	PS_colormask_off = 0, PS_colormask,*edge;
+
+	GMT_LONG bin, i, ij, k, k1, n, nm, nm2, nm_use, mx, my, n_drape = 0, bin_inc[4], ij_inc[4];
 
 	float *grd[3], *zgrd, *intensity = NULL, *topo;
 
@@ -150,16 +152,16 @@ int main (int argc, char **argv)
 	struct GRDVIEW_BIN *binij = NULL;
 	struct GRDVIEW_CTRL *Ctrl;
 
-	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, int *n);
-	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, int *n);
-	double get_z_ave (double v[], double next_up, int n);
-	void add_node (double x[], double y[], double z[], double v[], int *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], int ij, int bin);
-	void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity);
-	double get_intensity (float *intensity, int k, int nx);
-	struct GRDVIEW_CONT *get_cont_struct (int bin, struct GRDVIEW_BIN *binij, double value);
+	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
+	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
+	double get_z_ave (double v[], double next_up, GMT_LONG n);
+	void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin);
+	void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity, BOOLEAN outline);
+	double get_intensity (float *intensity, GMT_LONG k, int nx);
+	struct GRDVIEW_CONT *get_cont_struct (GMT_LONG bin, struct GRDVIEW_BIN *binij, double value);
 	struct GRDVIEW_POINT *get_point (double x, double y);
 	void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level);
-	int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[]);
+	int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[]);
 	int quick_idist(int x1, int y1, int x2, int y2);
 	int get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2);
 	BOOLEAN GMT_rgb_is_nan_rgb (int rgb[]);
@@ -172,7 +174,11 @@ int main (int argc, char **argv)
 	topofile = CNULL;
 	GMT_3D_mode = 1;	/* Only do background axis first; do foreground at end */
 	grd[0] = grd[1] = grd[2] = (float *)NULL;
-
+	GMT_grd_init (&header, argc, argv, FALSE);
+	GMT_grd_init (&t_head, argc, argv, FALSE);
+	GMT_grd_init (&i_head, argc, argv, FALSE);
+	for (i = 0; i < 3; i++) GMT_grd_init (&d_head[i], argc, argv, FALSE);
+	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -204,7 +210,7 @@ int main (int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
@@ -232,7 +238,7 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						Ctrl->L.active = TRUE;
-						strncpy (Ctrl->L.mode, &argv[i][2], 4);
+						strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					}
 					else
 						Ctrl->L.interpolant = BCR_BILINEAR;
@@ -262,7 +268,7 @@ int main (int argc, char **argv)
 						case 'm':	/* Mesh plot */
 							Ctrl->Q.mode = GRDVIEW_MESH;
 							if (argv[i][3] == '/' && GMT_getfill (&argv[i][4], &Ctrl->Q.fill)) {
-								fprintf (stderr, "%s: GMT SYNTAX ERROR -Qm option: To give mesh color, use -Qm<color>\n", GMT_program);
+								fprintf (stderr, "%s: GMT SYNTAX ERROR -Qm option: To give mesh color, use -Qm/<color>\n", GMT_program);
 								error = TRUE;
 							}
 							break;
@@ -270,7 +276,7 @@ int main (int argc, char **argv)
 							Ctrl->Q.mode = GRDVIEW_SURF;
 							if (argv[i][3] == 'm') Ctrl->Q.outline = TRUE;
 							break;
-						case 't':	/* textire image */
+						case 't':	/* texture image */
 							Ctrl->Q.special = TRUE;
 						case 'i':	/* image w/ clipmask */
 						case 'I':	/* Backward compatibility, gives -Qi */
@@ -279,6 +285,7 @@ int main (int argc, char **argv)
 							break;
 						case 'c':	/* image w/ colormask */
 							if (argv[i][3] && isdigit ((int)argv[i][3])) Ctrl->Q.dpi = atoi (&argv[i][3]);
+							Ctrl->Q.mode = GRDVIEW_IMAGE;
 							Ctrl->Q.mask = TRUE;
 							break;
 						default:
@@ -305,31 +312,31 @@ int main (int argc, char **argv)
 						}
 					}
 					break;
-				case 'W':	/* Contour and/or mesh pens */
+				case 'W':	/* Contour, mesh, or facade pens */
 					Ctrl->W.active = TRUE;
-					j = (argv[i][2] == 'm' || argv[i][2] == 'c') ? 3 : 2;
-                                        if (j == 2) {   /* Set both */
+					j = (argv[i][2] == 'm' || argv[i][2] == 'c' || argv[i][2] == 'f') ? 3 : 2;
+					if (j == 2) {   /* Set all */
 						if (GMT_getpen (&argv[i][2], &Ctrl->W.pen[0])) {
 							GMT_pen_syntax ('W', " ");
 							error++;
 						}
 						else
-							Ctrl->W.pen[1] = Ctrl->W.pen[0];
+							Ctrl->W.pen[2] = Ctrl->W.pen[1] = Ctrl->W.pen[0];
 						Ctrl->W.contour = TRUE;
 					}
 					else {	/* Here , j = 3.  BUT: First check that the m or c is not part of a color name instead */
 						char txt_a[GMT_LONG_TEXT];
 						n = j+1;
 						while (!(argv[i][n] == ',' || argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-						strncpy (txt_a, &argv[i][2], n-2);	txt_a[n-2] = '\0';
+						strncpy (txt_a, &argv[i][2], (size_t)(n-2));	txt_a[n-2] = '\0';
 						if (GMT_colorname2index (txt_a) >= 0) j = 2;	/* Found a colorname; reset j to 2 */
-						id = (argv[i][2] == 'm') ? 1 : 0;
+						id = (argv[i][2] == 'f') ? 2 : ((argv[i][2] == 'm') ? 1 : 0);
 						if (GMT_getpen (&argv[i][j], &Ctrl->W.pen[id])) {
 							GMT_pen_syntax ('W', " ");
 							error++;
 						}
 						if (j == 2) {	/* We discovered it wasnt -Wc or -Wm after all so copy pen */
-							Ctrl->W.pen[1] = Ctrl->W.pen[0];
+							Ctrl->W.pen[2] = Ctrl->W.pen[1] = Ctrl->W.pen[0];
 							Ctrl->W.contour = TRUE;
 						}
 						else if (id == 0)
@@ -359,7 +366,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdview %s - Plot topofiles in 3-D\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdview <topofile> %s [-B<tickinfo>] [-C<cpt_file>] [-E<azimuth/elevation>]\n", GMT_J_OPT);
+		fprintf (stderr, "usage: grdview <topofile> %s [-B<tickinfo>] [-C<cpt_file>] [%s]\n", GMT_J_OPT, GMT_E_OPT);
 		fprintf (stderr, "\t[-G<drapefile> | -G<grd_r>,<grd_g>,<grd_b>] [-I<intensfile>] [%s] [-K] [-L[<flags>]]\n", GMT_Jz_OPT);
 		fprintf (stderr, "\t[-N<level>[/<r/g/b>]] [-O] [-P] [-Q<type>[g]] [%s]\n", GMT_Rgeoz_OPT);
 		fprintf (stderr, "\t[-S<smooth>] [-T[s][o[<pen>]]] [%s] [-V] [-W<type><pen>]\n\t[%s] [%s] [-Z<z_level>] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
@@ -372,8 +379,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Color palette file\n");
-		fprintf (stderr, "\t-E Draw perspective from viewpoint at azimuth <v_az>, elevation <v_el> (degrees).\n");
-		fprintf (stderr, "\t   Default is looking straight down [180/90].\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-G <drapefile> rather than <topofile> is the data set to color-code.\n");
 		fprintf (stderr, "\t   Use <topofile> as the relief and \'drape\' the image on top.\n");
 		fprintf (stderr, "\t   Note that -Jz and -N always refers to the <topofile>\n");
@@ -393,7 +399,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q sets plot reQuest. Choose one of the following:\n");
-		fprintf (stderr, "\t   -Qm for Mesh plot [Default].  Append color for mesh paint [%d/%d/%d]\n", Ctrl->Q.fill.rgb[0], Ctrl->Q.fill.rgb[1], Ctrl->Q.fill.rgb[2]);
+		fprintf (stderr, "\t   -Qm for Mesh plot [Default].  Append /color for mesh paint [%d/%d/%d]\n", Ctrl->Q.fill.rgb[0], Ctrl->Q.fill.rgb[1], Ctrl->Q.fill.rgb[2]);
 		fprintf (stderr, "\t   -Qs[m] for colored or shaded Surface.  Append m to draw meshlines on the surface.\n");
 		fprintf (stderr, "\t   -Qi for scanline converting polygons to rasterimage.  Append effective dpi [100].\n");
 		fprintf (stderr, "\t   -Qc. As -Qi but use PS Level 3 colormasking for nodes with z = NaN.  Append effective dpi [100].\n");
@@ -401,17 +407,21 @@ int main (int argc, char **argv)
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S will smooth contours first (see grdcontour for <smooth> value info).\n");
 		GMT_pen_syntax ('T', "will image the data without interpolation by painting polygonal tiles\n\t   Append s to skip tiles for nodes with z = NaN [Default paints all tiles]\n\t   Append o[<pen>] to draw tile outline [Default uses no outline]");
+		fprintf (stderr, "\t   Cannot be used with -Jz|Z as it produces a flat image.\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		GMT_pen_syntax ('W', "sets pen attributes for contours in form <type><pen> [Default is no contours]");
-		fprintf (stderr, "\t   <type> can be c for contours or m for mesh\n");
+		GMT_pen_syntax ('W', "sets pen attributes for various features in form <type><pen>");
+		fprintf (stderr, "\t   <type> can be c for contours, m for mesh, and f for facade\n");
 		fprintf (stderr, "\t   c draw scontours on top of surface or mesh.  [Default is no contours]\n");
 		fprintf (stderr, "\t     Optionally append pen attributes [width = %gp, color = (%d/%d/%d), solid line].\n", 
 			Ctrl->W.pen[0].width, Ctrl->W.pen[0].rgb[0], Ctrl->W.pen[0].rgb[1], Ctrl->W.pen[0].rgb[2]);
 		fprintf (stderr, "\t   m sets attributes for mesh lines [[width = %gp, color = (%d/%d/%d), solid line].\n", 
 			Ctrl->W.pen[1].width, Ctrl->W.pen[1].rgb[0], Ctrl->W.pen[1].rgb[1], Ctrl->W.pen[1].rgb[2]);
 		fprintf (stderr, "\t     Requires -Qm or -Qsm to take effect.\n");
+		fprintf (stderr, "\t   f sets attributes for facade outline [[width = %gp, color = (%d/%d/%d), solid line].\n", 
+			Ctrl->W.pen[2].width, Ctrl->W.pen[2].rgb[0], Ctrl->W.pen[2].rgb[1], Ctrl->W.pen[2].rgb[2]);
+		fprintf (stderr, "\t     Requires -N to take effect.\n");
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
 		GMT_explain_option ('c');
@@ -456,6 +466,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Qi option:  Must specify positive dpi\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->T.active && project_info.JZ_set) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Cannot specify -JZ|z\n", GMT_program);
+		error++;
+	}
 	if (Ctrl->S.value < 0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  smooth value must be positive\n", GMT_program);
 		error++;
@@ -534,6 +548,7 @@ int main (int argc, char **argv)
 			GMT_map_basemap ();
 		GMT_plotend ();
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	/* Read data */
@@ -561,8 +576,8 @@ int main (int argc, char **argv)
 	ny = GMT_get_n (data_south, data_north, t_head.y_inc, t_head.node_offset);
 	mx = nx + 2 * two;
 	my = ny + 2 * two;
-	nm = nx * ny;
-	nm2 = mx * my;
+	nm = GMT_get_nm (nx, ny);
+	nm2 = GMT_get_nm (mx, my);
 	topo = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), GMT_program);
 	GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = two;
 	GMT_err_fail (GMT_read_grd (topofile, &t_head, topo, data_west, data_east, data_south, data_north, GMT_pad, FALSE), topofile);
@@ -733,7 +748,7 @@ int main (int argc, char **argv)
 		for (i = 0; i < t_head.nx - 1; i++, bin++, ij++) {
 			for (jj = n_out = 0; jj < 2; jj++) {	/* Loop over the 4 nodes making up one tile */
 				for (ii = 0; ii < 2; ii++) {
-					bin = (j + jj) * t_head.nx + (i + ii);
+					bin = GMT_IJ (j + jj, i + ii, t_head.nx);
 					ij = (two) ? ((j + jj) + 2) * mx + (i + ii) + two : bin;
 					if (GMT_is_fnan (topo[ij])) no_nans = FALSE;
 					if (GMT_map_outside (xval[i+ii], yval[j+jj])) n_out++;
@@ -767,48 +782,49 @@ int main (int argc, char **argv)
 	yy = (double *) GMT_memory (VNULL, (size_t)max, sizeof(double), GMT_program);
 	if (Ctrl->N.active) {
 		ps_comment ("Plot the plane at desired level");
+		GMT_setpen (&Ctrl->W.pen[2]);
 		if (!z_project.draw[0])	{	/* Southern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_min + dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[2])	{	/* Northern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_max - dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[3])	{	/* Western side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_min + delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
 			}
 		}
 		if (!z_project.draw[1])	{	/* Eastern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, 2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_max - delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
 			}
 		}
 
@@ -836,9 +852,10 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (Ctrl->T.active) {	/* Plot image as polygonal pieces */
-		int p;
-		double *xx, *yy, xpos, ypos;
+	if (Ctrl->T.active) {	/* Plot image as polygonal pieces. Here, -JZ is not set */
+		double *xx, *yy;
+		struct GMT_FILL fill;
+		GMT_init_fill (&fill, -1, -1, -1);	/* Initialize fill structure */
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling without interpolation\n", GMT_program);
 
@@ -846,28 +863,24 @@ int main (int argc, char **argv)
 		for (j = k = 0; j < header.ny; j++) {
 			for (i = 0; i < header.nx; i++, k++) {	/* Compute rgb for each pixel */
 				if (GMT_is_fnan (topo[k]) && Ctrl->T.skip) continue;
-				GMT_get_rgb_from_z (topo[k], rgb);
-				if (Ctrl->I.active) GMT_illuminate (intensity[k], rgb);
-
+				if (Ctrl->I.active && Ctrl->T.skip && GMT_is_fnan (intensity[k])) continue;
+				GMT_get_rgb_from_z (topo[k], fill.rgb);
+				if (Ctrl->I.active) GMT_illuminate (intensity[k], fill.rgb);
 				n = GMT_graticule_path (&xx, &yy, 1, xval[i] - dx2, xval[i] + dx2, yval[j] - dy2, yval[j] + dy2);
-
-				/* Do map projection and plot */
-
-				for (p = 0; p < n; p++) {
-					GMT_geoz_to_xy (xx[p], yy[p], topo[k], &xpos, &ypos);
-					xx[p] = xpos;	yy[p] = ypos;
-				}
-				ps_polygon (xx, yy, n, rgb, Ctrl->T.outline);
+				GMT_fill_polygon (xx, yy, 0.0, n, &fill, Ctrl->T.outline);
 				GMT_free ((void *)xx);
 				GMT_free ((void *)yy);
 			}
 		}
+		GMT_free ((void *) xval);
+		GMT_free ((void *) yval);
 	}
+	
 	if (Ctrl->Q.mode == GRDVIEW_IMAGE) {	/* compute image */
-		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, *ix, *iy;
+		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, last_i, last_j, *ix, *iy;
 		int *top_jp = VNULL, *bottom_jp = VNULL, p;
 		BOOLEAN done;
-		double xp, yp, sum_w, w, sum_i, x_width, y_width, value, i_dpi_i;
+		double xp, yp, sum_w, w, sum_i, x_width, y_width, value;
 		double sum_r, sum_g, sum_b, intval = 0.0, y_drape, *x_drape;
 		float *int_drape = VNULL;
 		unsigned char *bitimage_24 = (unsigned char *)NULL, *bitimage_8 = (unsigned char *)NULL;
@@ -883,7 +896,7 @@ int main (int argc, char **argv)
 		y_width = z_project.ymax - z_project.ymin;
 		nx_i = irint (x_width * Ctrl->Q.dpi);	/* Size of image in pixels */
 		ny_i = irint (y_width * Ctrl->Q.dpi);
-		i_dpi_i = 1.0 / Ctrl->Q.dpi;
+		last_i = nx_i - 1;	last_j = ny_i - 1;
 		if (drape_resample) {
 			nm_drape = d_head[0].nx * d_head[0].ny;
 			ix = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
@@ -900,8 +913,9 @@ int main (int argc, char **argv)
 					}
 					else {
 						GMT_geoz_to_xy (x_drape[i], y_drape, value, &xp, &yp);
-						ix[bin] = (int)floor ((xp - z_project.xmin) * Ctrl->Q.dpi);
-						iy[bin] = (int)floor ((yp - z_project.ymin) * Ctrl->Q.dpi);
+						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
+						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 					if (Ctrl->I.active) int_drape[bin] = (float)GMT_get_bcr_z(&i_head, x_drape[i], y_drape, intensity, &edgeinfo, &i_bcr);
 				}
@@ -923,8 +937,9 @@ int main (int argc, char **argv)
 					}
 					else {
 						GMT_geoz_to_xy (xval[i], yval[j], (double)topo[ij], &xp, &yp);
-						ix[bin] = (int)floor ((xp - z_project.xmin) * Ctrl->Q.dpi);
-						iy[bin] = (int)floor ((yp - z_project.ymin) * Ctrl->Q.dpi);
+						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
+						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 				}
 			}
@@ -971,7 +986,7 @@ int main (int argc, char **argv)
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Start rasterization\n", GMT_program);
 		for (j = j_start; j != j_stop; j += j_inc) {
 
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.4d\r", GMT_program, j);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.6d\r", GMT_program, j);
 
 			for (i = i_start; i != i_stop; i += i_inc) {
 				bin = j * header.nx + i;
@@ -1011,7 +1026,10 @@ int main (int argc, char **argv)
 							}
 							else {		/* Use lookup to get color */
 								GMT_get_rgb_from_z (zgrd[node], rgb);
-								if (GMT_is_fnan (zgrd[node])) bad++;	/* watch out for NaNs */
+								if (GMT_is_fnan (zgrd[node]))	/* watch out for NaNs in the z-data*/
+									bad++;
+								else if (Ctrl->I.active && GMT_is_fnan (intensity[node]))	/* watch out for NaNs in the intensity data*/
+									bad++;
 							}
 							if (bad) continue;	/* We don't want to blend in the (typically) gray NaN colors with the others. */
 							
@@ -1073,7 +1091,7 @@ int main (int argc, char **argv)
 					y_imask[nk-k] = y_imask[nk-k1] = z_project.ymin + bottom_jp[ip] * y_pixel_size;
 				}
 			}
-			ps_clipon (x_imask, y_imask, 4 * nx_i, GMT_no_rgb, 3);
+			ps_clipon (x_imask, y_imask, (GMT_LONG)(4 * nx_i), GMT_no_rgb, 3);
 			GMT_free ((void *)x_imask);
 			GMT_free ((void *)y_imask);
 		}
@@ -1118,7 +1136,7 @@ int main (int argc, char **argv)
 				GMT_geoz_to_xy (x_right, y_bottom, (double)(topo[ij+ij_inc[1]]), &xx[1], &yy[1]);
 				GMT_geoz_to_xy (x_right, y_top, (double)(topo[ij+ij_inc[2]]), &xx[2], &yy[2]);
 				GMT_geoz_to_xy (x_left, y_top, (double)(topo[ij+ij_inc[3]]), &xx[3], &yy[3]);
-				ps_patch (xx, yy, 4, Ctrl->Q.fill.rgb, TRUE);
+				ps_patch (xx, yy, (GMT_LONG)4, Ctrl->Q.fill.rgb, TRUE);
 				if (Ctrl->W.contour) {
 					pen_not_set = TRUE;
 					if (binij[bin].first_cont == NULL) continue;
@@ -1133,7 +1151,7 @@ int main (int argc, char **argv)
 							GMT_setpen (&Ctrl->W.pen[0]);
 							pen_not_set = FALSE;
 						}
-						ps_line (xx, yy, k, 3, FALSE, TRUE);
+						ps_line (xx, yy, (GMT_LONG)k, 3, FALSE, TRUE);
 					}
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 				}
@@ -1170,8 +1188,7 @@ int main (int argc, char **argv)
 					X_vert[0] = X_vert[3] = x_left;	X_vert[1] = X_vert[2] = x_right;
 					Y_vert[0] = Y_vert[1] = y_bottom;	Y_vert[2] = Y_vert[3] = y_top;
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], 0.0, &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, 4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0, Ctrl->Q.outline);
 					continue;
 				}
 
@@ -1284,7 +1301,7 @@ int main (int argc, char **argv)
 
 								/* Now paint the polygon piece */
 
-								paint_it (xx, yy, n, z_ave-saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+								paint_it (xx, yy, n, z_ave-saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 								/* Reset the anchor points to previous contour */
 
@@ -1316,7 +1333,7 @@ int main (int argc, char **argv)
 
 							/* Now paint the polygon piece */
 
-							paint_it (xx, yy, n, z_ave+saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+							paint_it (xx, yy, n, z_ave+saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 						} /* End triangular piece */
 
@@ -1382,7 +1399,7 @@ int main (int argc, char **argv)
 
 							/* Now paint the polygon piece */
 
-							paint_it (xx, yy, n, z_ave-small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+							paint_it (xx, yy, n, z_ave-small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 							/* Reset the anchor points to previous contour */
 
@@ -1411,7 +1428,7 @@ int main (int argc, char **argv)
 
 						/* Now paint the polygon piece */
 
-						paint_it (xx, yy, n, z_ave+small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+						paint_it (xx, yy, n, z_ave+small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 					} /* End non-saddle case */
 
@@ -1435,7 +1452,7 @@ int main (int argc, char **argv)
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 					if (Ctrl->Q.outline) {
 						for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
-						ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+						ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
 					}
 				}
 				else {	/* No Contours, paint tile (any of the corner values will give correct color */
@@ -1449,8 +1466,7 @@ int main (int argc, char **argv)
 					/* Now paint the polygon piece */
 
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, 4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, Ctrl->Q.outline);
 				}
 			}
 		}
@@ -1468,7 +1484,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->N.facade) {	/* Cover the two front sides */
 		ps_comment ("Painting the frontal facade");
-		GMT_setpen (&Ctrl->W.pen[0]);
+		GMT_setpen (&Ctrl->W.pen[2]);
 		if (!z_project.draw[0])	{	/* Southern side */
 			for (i = n = 0, ij = sw; i < header.nx; i++, ij++) {
 				if (GMT_is_fnan (topo[ij])) continue;
@@ -1555,7 +1571,7 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-struct GRDVIEW_CONT *get_cont_struct (int bin, struct GRDVIEW_BIN *binij, double value)
+struct GRDVIEW_CONT *get_cont_struct (GMT_LONG bin, struct GRDVIEW_BIN *binij, double value)
 {
 	struct GRDVIEW_CONT *cont, *new_cont;
 
@@ -1585,7 +1601,8 @@ struct GRDVIEW_POINT *get_point (double x, double y)
 
 void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level)
 {
-	int i, j, ij;
+	int i, j;
+	GMT_LONG ij;
 	double xtmp, ytmp, tmp, delx, dely;
 
 	delx = (header->node_offset) ? 0.5 * header->x_inc :0.0;
@@ -1595,7 +1612,7 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 	z_project.ymax = z_project.ymin;	/* Reset from whatever it was */
 
 	for (j = 0; j < header->ny; j++) {
-		ij = (j + two) * header->nx + two;
+		ij = GMT_IJ (j + two, two, header->nx);
 		for (i = 0; i < header->nx; i++, ij++) {
 			if (GMT_is_fnan (topo[ij])) continue;
 			GMT_geoz_to_xy (header->x_min + i * header->x_inc, header->y_max - j * header->y_inc, (double)topo[ij], &xtmp, &ytmp);
@@ -1625,13 +1642,13 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 	}
 }
 
-double get_intensity (float *intensity, int k, int nx)
+double get_intensity (float *intensity, GMT_LONG k, int nx)
 {
 	/* Finds the average intensity for this polygon */
 	return (0.25 * (intensity[k] + intensity[k+1] + intensity[k-nx] + intensity[k-nx+1]));
 }
 
-int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[])
+int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[])
 {
 	int i, what;
 	double x[6], y[6];
@@ -1641,7 +1658,7 @@ int pixel_inside (int ip, int jp, int *ix, int *iy, int bin, int bin_inc[])
 		y[i] = iy[bin+bin_inc[i]];
 	}
 	x[4] = x[0];	y[4] = y[0];
-	what = GMT_non_zero_winding ((double)ip, (double)jp, x, y, 5);
+	what = GMT_non_zero_winding ((double)ip, (double)jp, x, y, (GMT_LONG)5);
 	return (what);
 }
 
@@ -1669,7 +1686,7 @@ int get_side (double x, double y, double x_left, double y_bottom, double xinc, d
 	return (side);
 }
 
-void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, int *n) {
+void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
 	int k;
 	for (k = 0; k < ncont; k++, (*n)++) {
 		x[*n] = xcont[k];
@@ -1679,7 +1696,7 @@ void copy_points_fw (double x[], double y[], double z[], double v[], double xcon
 	}
 }
 
-void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, int *n) {
+void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
 	int k;
 	for (k = ncont - 1; k >= 0; k--, (*n)++) {
 		x[*n] = xcont[k];
@@ -1689,15 +1706,15 @@ void copy_points_bw (double x[], double y[], double z[], double v[], double xcon
 	}
 }
 
-double get_z_ave (double v[], double next_up, int n) {
-	int k;
+double get_z_ave (double v[], double next_up, GMT_LONG n) {
+	GMT_LONG k;
 	double z_ave;
 
 	for (k = 0, z_ave = 0.0; k < n; k++) z_ave += MIN (v[k], next_up);
 	return (z_ave / n);
 }
 
-void add_node (double x[], double y[], double z[], double v[], int *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], int ij, int bin) {
+void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin) {
 	/* Adds a corner node to list of points and increments counter */
 	x[*k] = X_vert[node];
 	y[*k] = Y_vert[node];
@@ -1706,7 +1723,7 @@ void add_node (double x[], double y[], double z[], double v[], int *k, int node,
 	(*k)++;
 }
 
-void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity) {
+void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity, BOOLEAN outline) {
 	int index, rgb[3];
 	struct GMT_FILL *f;
 
@@ -1718,12 +1735,12 @@ void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, BOOLEAN
 	/* Now we must paint, with colors or patterns */
 
 	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {	/* Pattern */
-		GMT_fill (x, y, n, f, FALSE);
+		GMT_fill (x, y, n, f, outline);
 	}
 	else {	/* Solid color/gray */
 		if (intens) GMT_illuminate (intensity, rgb);
 		if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb); /* GMT_YIQ transformation */
-		ps_patch (x, y, n, rgb, FALSE);	/* Contours drawn separately (after this call) if desired */
+		ps_patch (x, y, n, rgb, outline);	/* Contours drawn separately (after this call) if desired */
 	}
 }
 
@@ -1735,7 +1752,7 @@ BOOLEAN GMT_rgb_is_nan_rgb (int rgb[])
 void *New_Grdview_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVIEW_CTRL *C;
 	
-	C = (struct GRDVIEW_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVIEW_CTRL), "New_Grdview_Ctrl");
+	C = (struct GRDVIEW_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDVIEW_CTRL), "New_Grdview_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -1743,6 +1760,7 @@ void *New_Grdview_Ctrl () {	/* Allocate and initialize a new control structure *
 	GMT_init_pen (&C->T.pen, GMT_PENWIDTH);
 	GMT_init_pen (&C->W.pen[0], 3.0 * GMT_PENWIDTH);	/* Contour pen */
 	GMT_init_pen (&C->W.pen[1], GMT_PENWIDTH);	/* Mesh pen */
+	GMT_init_pen (&C->W.pen[2], 3.0 * GMT_PENWIDTH);	/* Facade pen */
 	C->Q.dpi = 100;
 	GMT_init_fill (&C->Q.fill, 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0]);
 	C->L.interpolant = BCR_BICUBIC; C->L.threshold = 1.0;
