@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pslib.h,v 1.47 2008/03/30 01:50:05 guru Exp $
+ *	$Id: pslib.h,v 1.54 2009/01/12 04:25:57 remko Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -87,10 +87,10 @@ struct EPS {    /* Holds info for eps files */
  	  to -1.  E.g., if 4 fonts are used, font[0], font[1], font[2], and
 	  font[3] must contain the integer ID of these fonts; font[4] = -1
 */
-	PS_LONG x0, x1, y0, y1;		/* Bounding box values in points */
+	double x0, x1, y0, y1;		/* Bounding box values in points */
 	int portrait;			/* TRUE if start of plot was portrait */
 	int clip_level;			/* Add/sub 1 as we clip/unclip - should end at 0 */
-    int fontno[PSL_MAX_EPS_FONTS];	/* Array with font ids used (skip if -1). 6 is max fonts used in GMT anot/labels */
+	int fontno[PSL_MAX_EPS_FONTS];	/* Array with font ids used (skip if -1). 6 is max fonts used in GMT anot/labels */
 	char *name;			/* User name */
 	char *title;			/* Plot title */
 };
@@ -120,10 +120,9 @@ struct imageinfo {
 
 /* Public functions */
 
-EXTERN_MSC PS_LONG ps_line (double *x, double *y, PS_LONG n, int type, int close, int split);
-EXTERN_MSC int ps_plotinit (char *plotfile, int overlay, int mode, double xoff, double yoff, double xscl, double yscl, int ncopies, int dpi, int unit, int *page_width, int *rgb, const char *encoding, struct EPS *eps);
 EXTERN_MSC void ps_arc (double x, double y, double radius, double az1, double az2, int status);
 EXTERN_MSC void ps_axis (double x, double y, double length, double val0, double val1, double annotation_int, char *label, double annotpointsize, int side);
+EXTERN_MSC void ps_bitimage (double x, double y, double xsize, double ysize, unsigned char *buffer, int nx, int ny, int invert, int f_rgb[], int b_rgb[]);
 EXTERN_MSC void ps_circle (double x, double y, double size, int rgb[], int outline);
 EXTERN_MSC void ps_clipoff (void);
 EXTERN_MSC void ps_clipon (double *x, double *y, PS_LONG n, int rgb[], int flag);
@@ -138,18 +137,22 @@ EXTERN_MSC void ps_diamond (double x, double y, double side, int rgb[], int outl
 EXTERN_MSC void ps_ellipse (double x, double y, double angle, double major, double minor, int rgb[], int outline);
 EXTERN_MSC void ps_epsimage (double x, double y, double xsize, double ysize, unsigned char *buffer, PS_LONG size, int nx, int ny, PS_LONG ox, PS_LONG oy); 
 EXTERN_MSC void ps_flush (void);
+EXTERN_MSC void *ps_memory (void *prev_addr, size_t nelem, size_t size);
 EXTERN_MSC void ps_free (void *addr);
 EXTERN_MSC void ps_hexagon (double x, double y, double side, int rgb[], int outline);
 EXTERN_MSC void ps_image (double x, double y, double xsize, double ysize, unsigned char *buffer, int nx, int ny, int nbits);
-EXTERN_MSC int ps_pattern (int image_no, char *imagefile, int invert, int image_dpi, int outline, int f_rgb[], int b_rgb[]);
-EXTERN_MSC void ps_bitimage (double x, double y, double xsize, double ysize, unsigned char *buffer, int nx, int ny, int invert, int f_rgb[], int b_rgb[]);
+EXTERN_MSC PS_LONG ps_line (double *x, double *y, PS_LONG n, int type, int close, int split);
 EXTERN_MSC void ps_itriangle (double x, double y, double side, int rgb[], int outline);
 EXTERN_MSC void ps_octagon (double x, double y, double side, int rgb[], int outline);
 EXTERN_MSC void ps_pentagon (double x, double y, double side, int rgb[], int outline);
+EXTERN_MSC int ps_pattern (int image_no, char *imagefile, int invert, int image_dpi, int outline, int f_rgb[], int b_rgb[]);
 EXTERN_MSC void ps_pie (double x, double y, double radius, double az1, double az2, int rgb[], int outline);
 EXTERN_MSC void ps_plot (double x, double y, int pen);
+EXTERN_MSC int ps_plotinit (char *plotfile, int overlay, int mode, double xoff, double yoff, double xscl, double yscl, int ncopies, int dpi, int unit, int *page_size, int *rgb, const char *encoding, struct EPS *eps);
+EXTERN_MSC int ps_plotinit_hires (char *plotfile, int overlay, int mode, double xoff, double yoff, double xscl, double yscl, int ncopies, int dpi, int unit, double *page_size, int *rgb, const char *encoding, struct EPS *eps);
 EXTERN_MSC void ps_plotend (int lastpage);
 EXTERN_MSC void ps_plotr (double x, double y, int pen);
+EXTERN_MSC void ps_plus (double x, double y, double size);
 EXTERN_MSC void ps_point (double x, double y, double diameter);
 EXTERN_MSC void ps_polygon (double *x, double *y, PS_LONG n, int rgb[], int outline);
 EXTERN_MSC void ps_rect (double x1, double y1, double x2, double y2, int rgb[], int outline);
@@ -158,6 +161,7 @@ EXTERN_MSC void ps_patch (double *x, double *y, PS_LONG np, int rgb[], int outli
 EXTERN_MSC void ps_rotatetrans (double x, double y, double angle);
 EXTERN_MSC void ps_segment (double x0, double y0, double x1, double y1);
 EXTERN_MSC void ps_setdash (char *pattern, int offset);
+EXTERN_MSC void ps_setfill (int rgb[], int outline);
 EXTERN_MSC void ps_setfont (int font_no);
 EXTERN_MSC void ps_setformat (int n_decimals);
 EXTERN_MSC void ps_setline (int linewidth);
@@ -177,7 +181,7 @@ EXTERN_MSC void ps_vector (double xtail, double ytail, double xtip, double ytip,
 EXTERN_MSC unsigned char *ps_load_image (char *file, struct imageinfo *header);
 EXTERN_MSC void ps_words (double x, double y, char **text, PS_LONG n_words, double line_space, double par_width, int par_just, int font, double font_size, double angle, int rgb[3], int justify, int draw_box, double x_off, double y_off, double x_gap, double y_gap, int boxpen_width, char *boxpen_texture, int boxpen_offset, int boxpen_rgb[], int vecpen_width, char *vecpen_texture, int vecpen_offset, int vecpen_rgb[], int boxfill_rgb[3]);
 EXTERN_MSC void ps_setline (int linewidth);
-EXTERN_MSC void ps_textdim (char *xdim, char *ydim, double pointsize, int font, char *text, int key);
+EXTERN_MSC void ps_textdim (char *dim, double pointsize, int font, char *text);
 EXTERN_MSC void ps_set_length (char *param, double value);
 EXTERN_MSC void ps_set_height (char *param, double fontsize);
 EXTERN_MSC void ps_define_rgb (char *param, int rgb[]);
