@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxy.c,v 1.145 2008/05/01 03:00:39 guru Exp $
+ *	$Id: psxy.c,v 1.151 2009/01/11 02:52:38 remko Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,6 +56,10 @@ struct PSXY_CTRL {
 		BOOLEAN active;
 		struct GMT_FILL fill;
 	} G;
+	struct I {	/* -I<intensity> */
+		BOOLEAN active;
+		double value;
+	} I;
 	struct L {	/* -L */
 		BOOLEAN active;
 	} L;
@@ -174,7 +178,7 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					break;
 				case 'D':
-					if ((j = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b)) >= 1) {
+					if ((j = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b)) < 1) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give x [and y] offsets\n", GMT_program);
 						error++;
 					}
@@ -225,6 +229,10 @@ int main (int argc, char **argv)
 						error++;
 					}
 					break;
+				case 'I':	/* Adjust symbol color via intensity */
+					Ctrl->I.value = atof (&argv[i][2]);
+					Ctrl->I.active = TRUE;
+					break;
 				case 'L':		/* Close line segments */
 					Ctrl->L.active = TRUE;
 					break;
@@ -264,7 +272,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"psxy %s - Plot lines, polygons, and symbols on maps\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: psxy <infiles> %s %s [-A[m|p]] [%s] [-C<cpt>]\n", GMT_J_OPT, GMT_Rgeo_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-D<dx>/<dy>] [-E[x|y|X|Y][n][cap][/[+|-]<pen>]] [-G<fill>] [%s] [-K] [-L] [%s]\n", GMT_H_OPT, GMT_Mo_OPT);
+		fprintf (stderr, "\t[-D<dx>/<dy>] [-E[x|y|X|Y][n][cap][/[+|-]<pen>]] [-G<fill>] [%s] [-I<intens>] [-K] [-L] [%s]\n", GMT_H_OPT, GMT_Mo_OPT);
 		fprintf (stderr, "\t[-N] [-O] [-P] [-S[<symbol>][<size>]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
 		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [%s]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 
@@ -284,13 +292,14 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-E means draw error bars for x, y, or both.  Add cap-width [%g]. Append pen attributes;\n", Ctrl->E.size);
 		fprintf (stderr, "\t   leading + applies cpt color (-C) to both symbol fill and error pen, - applies it to pen only.\n");
 		fprintf (stderr, "\t   If X or Y is used instead a box-and-whisker diagram is drawn instead, using data from 4\n");
-		fprintf (stderr, "\t   extra columns to get the 0 %%, 25 %%, 75 %% ,and 100%% quartiles (point value is assumed to be 50%%)\n");
+		fprintf (stderr, "\t   extra columns to get the 0 %%, 25 %%, 75 %% ,and 100%% quantiles (point value is assumed to be 50%%)\n");
 		fprintf (stderr, "\t   If n is appended after X (or Y) we expect a 5th extra column with the sample size which is needed to draw\n");
 		fprintf (stderr, "\t   a notched box-and whisker diagram (notch width represents uncertainty in median).\n");
 		fprintf (stderr, "\t   Finally, use -W, -G to control the appearance of the 25-75%% box.\n");
 		GMT_fill_syntax ('G', "Specify color or pattern [Default is no fill].");
 		fprintf (stderr, "\t   -G option can be present in all subheaders (requires -M and no -S)\n");
 		GMT_explain_option ('H');
+		fprintf (stderr, "\t-I sets a constant intensity used to modulate the fill color (requires -C or -G)\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L Force closed polygons.\n");
 		GMT_explain_option ('M');
@@ -298,7 +307,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-S to select symbol type and symbol size (in %s).  Choose between\n", GMT_unit_names[gmtdefs.measure_unit]);
-		fprintf (stderr, "\t   -(xdash), st(a)r, (b|B)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, octa(g)on, (h)exagon, (i)nvtriangle,\n");
+		fprintf (stderr, "\t   -(xdash), +(plus), st(a)r, (b|B)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, octa(g)on, (h)exagon, (i)nvtriangle,\n");
 		fprintf (stderr, "\t   (j)rotated rectangle, (k)ustom, (l)etter, pe(n)tagon, (p)oint, (q)uoted line, (r)ect, (s)quare,\n");
 		fprintf (stderr, "\t   (t)riangle, (v)ector, (w)edge, (x)cross, (y)dash\n");
 		fprintf (stderr, "\t   If no size is specified, psxy expects the 3rd column to have sizes.\n");
@@ -308,8 +317,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Bars: Append b<base> to give the y-value of the base [Default = 0]\n");
 		fprintf (stderr, "\t      Append u if width is in x-input units [Default is %s]\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t      Use upper case -SB for horizontal bars (base then refers to x and width may be in y-units [Default is vertical]\n");
-		fprintf (stderr, "\t   Ellipses: the direction, major, and minor axis must be in input columns 3, 4 and 5.\n");
-		fprintf (stderr, "\t     If -SE rather than -Se is selected, psxy will expect azimuth, and axes in km\n");
+		fprintf (stderr, "\t   Ellipses: the direction, semi-major, and semi-minor axis must be in input columns 3, 4 and 5.\n");
+		fprintf (stderr, "\t     If -SE rather than -Se is selected, psxy will expect azimuth, and semi-axes in km\n");
 		fprintf (stderr, "\t     and convert azimuths based on the chosen map projection.  If projection is linear then\n");
 		fprintf (stderr, "\t     we scale the minor/major axes by the map scale.\n");
 		fprintf (stderr, "\t   Rotatable Rectangle: Direction, x-dimension, and y-dimension are expected in columns 3, 4, and 5.\n");
@@ -443,6 +452,10 @@ int main (int argc, char **argv)
 	current_pen = default_pen = Ctrl->W.pen;
 	current_fill = default_fill = Ctrl->G.fill;
 	default_outline = Ctrl->W.active;
+	if (Ctrl->I.active) {
+		GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+		GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+	}
 
 	Ctrl->E.size *= 0.5;	/* Since we draw half-way in either direction */
 
@@ -577,6 +590,10 @@ int main (int argc, char **argv)
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
 					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
+					if (Ctrl->I.active) {
+						GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+						GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+					}
 					if (change & 4) GMT_setpen (&current_pen);
 
 					if (read_symbol) n_expected = BUFSIZ;
@@ -607,7 +624,8 @@ int main (int argc, char **argv)
 
 				if (get_rgb) {
 					GMT_get_rgb_from_z (in[2], current_fill.rgb);
-					if (S.symbol == GMT_SYMBOL_CROSS || S.symbol == GMT_SYMBOL_POINT || S.symbol == GMT_SYMBOL_TEXT) ps_setpaint (current_fill.rgb);
+					if (Ctrl->I.active) GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+					if (S.symbol == GMT_SYMBOL_CROSS || S.symbol == GMT_SYMBOL_PLUS || S.symbol == GMT_SYMBOL_POINT || S.symbol == GMT_SYMBOL_TEXT) ps_setpaint (current_fill.rgb);
 				}
 				if (GMT_cpt_skip) continue;	/* Chosen cpt file indicates skip for this z */
 		
@@ -757,6 +775,9 @@ int main (int argc, char **argv)
 					case GMT_SYMBOL_CROSS:
 						ps_cross (plot_x, plot_y, S.size_x);
 						break;
+					case GMT_SYMBOL_PLUS:
+						ps_plus (plot_x, plot_y, S.size_x);
+						break;
 					case GMT_SYMBOL_POINT:
 						if (reset_cap == 2) ps_setlinecap (1);
 						ps_point (plot_x, plot_y, S.size_x);
@@ -920,6 +941,10 @@ int main (int argc, char **argv)
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
 					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
+					if (Ctrl->I.active) {
+						GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+						GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+					}
 					if ((change & 4) && penset_OK) GMT_setpen (&current_pen);
 					if ((change & 1)) polygon = TRUE;
 					if ((change & 2) && !Ctrl->L.active) {
